var calculator = 
{
	getHyperFocalDistance: function (focal_length, circle_of_confusion, f_stop, to_units)
	{
		// check for valid values
		if(focal_length <= 0 || circle_of_confusion <= 0 || f_stop <= 0)
			return 0;
			
		var hfd_mm = ((focal_length * focal_length) / (circle_of_confusion * f_stop)) + focal_length; 
		return units.fromMilimeters(hfd_mm, to_units);
	},
	
	getDepthOfField: function (focal_length, circle_of_confusion, f_stop, subject_distance, to_units)
	{
		var subject_distance_mm = units.toMilimeters(subject_distance, to_units);
		var hyper_focal_distance_mm = this.getHyperFocalDistance(focal_length, circle_of_confusion, f_stop, 'milimeters');
		
		var distance_near_mm = (subject_distance_mm * (hyper_focal_distance_mm - focal_length)) / (hyper_focal_distance_mm + subject_distance_mm - focal_length - focal_length); 
		var distance_far_mm = "infinite";
		var dof_mm = "infinite";
		var dof_mm_actual = 0;
		var in_front_of_subject_mm = subject_distance_mm - distance_near_mm;
		var behind_subject_mm = "infinite";

		distance_far_mm = (subject_distance_mm * (hyper_focal_distance_mm - focal_length)) / Math.abs(hyper_focal_distance_mm - subject_distance_mm);
		dof_mm_actual = distance_far_mm - distance_near_mm;
	
		if(distance_far_mm - subject_distance_mm > 100000)
		{
			distance_far_mm = "infinite";
			dof_mm = "infinite";
			behind_subject_mm = "infinite";
		}
		else if (subject_distance_mm <= focal_length)
		{
			distance_far_mm = 'N/A';
			distance_near_mm = 'N/A'
			dof_mm = 'N/A';
			behind_subject_mm = 'N/A';
			in_front_of_subject_mm = 'N/A';
		}
		else
		{
			dof_mm = distance_far_mm - distance_near_mm;
			behind_subject_mm = distance_far_mm - subject_distance_mm;
		}
	
		var result = new Array();
		
		result["focusDistance"]	= units.fromMilimeters(subject_distance_mm, to_units);
		result["nearLimit"]		= units.fromMilimeters(distance_near_mm, to_units);
		result["farLimit"]		= units.fromMilimeters(distance_far_mm, to_units);
		result["depthOfField"]	= units.fromMilimeters(dof_mm, to_units);

		result["inFrontOfSubject"]	= units.fromMilimeters(in_front_of_subject_mm, to_units);
		result["behindSubject"]		= units.fromMilimeters(behind_subject_mm, to_units);

		result["inFrontOfSubjectPercent"]	= ((in_front_of_subject_mm / dof_mm_actual) * 100).toFixed(2) + '%';
		result["behindSubjectPercent"]		= (((dof_mm_actual - in_front_of_subject_mm) / dof_mm_actual) * 100).toFixed(2) + '%';

		for (key in result)
		{
			if (!isNaN(result[key]))
			{
				result[key] = (result[key]).toFixed(4) + " " + units.short_names[to_units];
			}
		}
		
		return result;
	}
}

var units = 
{
	milimeters: [],
	short_names: [],
	init: function ()
	{
		this.milimeters['milimeters'] = 1;
		this.milimeters['centimeters'] = 10;
		this.milimeters['meters'] = 1000;
		this.milimeters['feet'] = 304.8;
		this.milimeters['inches'] = 25.4;
		
		this.short_names['milimeters'] = "mm";
		this.short_names['centimeters'] = "cm";
		this.short_names['meters'] = "m";
		this.short_names['feet'] = "ft";
		this.short_names['inches'] = "inch";
	},
	toMilimeters: function (value, unit)
	{
		return value * this.milimeters[unit];
	},
	fromMilimeters: function (value, unit)
	{
		if (isNaN(value)) 
		{
			return value;
		}
		else
		{
			return value / this.milimeters[unit];
		}
	}
}
